[System.Reflection.Assembly]::LoadWithPartialName('Microsoft.VisualBasic') | Out-Null
$tenantName = [Microsoft.VisualBasic.Interaction]::InputBox("Enter Azure AD domain name (e.g. yourdomain.com)", "Domain")
#$tenantName = "wanpathdev.onmicrosoft.com"
$ReplyURL = "https://www.myworkdrive.com/azure-b2b-guest-app-success/"
$DisplayName = [Microsoft.VisualBasic.Interaction]::InputBox("Enter Application Display Name (e.g. MWD Guest App)", "DisplayName")
#Set TLS 1.2
[Net.ServicePointManager]::SecurityProtocol = [Net.SecurityProtocolType]::Tls12

# Install NuGet
Install-PackageProvider -Name NuGet -MinimumVersion 2.8.5.201 -Force

#install the Azure AD module
Install-Module -Name AzureAD

#connect to the Azure
$azureAd = Connect-AzureAD -TenantId $tenantName

#create a new application
New-AzureADApplication -DisplayName $DisplayName -ReplyUrls $ReplyURL
$mwdApp = Get-AzureADApplication -Filter "DisplayName eq '$DisplayName'"

#add a new access key
$mwdAppAccessKey = New-AzureADApplicationPasswordCredential -ObjectId $mwdApp.ObjectId -CustomKeyIdentifier "Access Key" -EndDate (get-date).AddYears(5)

#grant the permissions
$api = Get-AzureADServicePrincipal -Filter "servicePrincipalNames/any(n:n eq 'https://graph.microsoft.com')"
$req = New-Object -TypeName "Microsoft.Open.AzureAD.Model.RequiredResourceAccess"
#Read directory data
$acc1 = New-Object -TypeName "Microsoft.Open.AzureAD.Model.ResourceAccess" -ArgumentList "7ab1d382-f21e-4acd-a863-ba3e13f7da61","Role"
#Invite guest users to the organization
$acc2 = New-Object -TypeName "Microsoft.Open.AzureAD.Model.ResourceAccess" -ArgumentList "09850681-111b-4a89-9bed-3f2cae46d706","Role"
#Read and write all users' full profiles
$acc3 = New-Object -TypeName "Microsoft.Open.AzureAD.Model.ResourceAccess" -ArgumentList "741f803b-c850-494e-b5df-cde7c675a1ca","Role"
#GroupMember.ReadWrite.All: Read and write all group memberships
$acc4 = New-Object -TypeName "Microsoft.Open.AzureAD.Model.ResourceAccess" -ArgumentList "dbaae8cf-10b5-4b86-a4a1-f871c94c6695","Role" 
$req.ResourceAccess = $acc1,$acc2,$acc3,$acc4
$req.ResourceAppId = $api.AppId
Set-AzureADApplication -ObjectId $mwdApp.ObjectId -RequiredResourceAccess $req

#sleep the script to allow Azure create new app
"Sleep to allow Azure create new app (10 seconds)"
Start-Sleep -Seconds 10

#connect again to the Azure
$azureAd = Connect-AzureAD -TenantId $tenantName

#grant the admin consent
$adminConsentUrl = "https://login.microsoftonline.com/" + $azureAd.TenantId.Guid + "/oauth2/authorize?client_id=" + $mwdApp.AppId + "&response_type=code&prompt=admin_consent"
Start-Process -FilePath $adminConsentUrl

#display the application id, the application tenant id, and the application secret
"Application Id: " + $mwdApp.AppId
"Application Tenant Id: " + $azureAd.TenantId.Guid
"Application Secret: " + $mwdAppAccessKey.value

#update configuration
$generalXml = "C:\Wanpath\WanPath.Data\Settings\General\General.xml"
$mwdAppId = $mwdApp.AppId
$azureAdTenantIdGuid = $azureAd.TenantId.Guid
$mwdAppAccessKeyValue = $mwdAppAccessKey.value


#update configuration - ExternalGuestUsersAzureAppID
$sel = Select-String -Path $generalXml "<ExternalGuestUsersAzureAppID>.*</ExternalGuestUsersAzureAppID>"
if ($sel -ne $null)
{
  #update
  $oldValue = "<ExternalGuestUsersAzureAppID>.*</ExternalGuestUsersAzureAppID>"
  $newValue = "<ExternalGuestUsersAzureAppID>" + $mwdAppId + "</ExternalGuestUsersAzureAppID>"
  ((Get-Content -path $generalXml -Raw) -replace $oldValue,$newValue) | Set-Content -Path $generalXml
}
else
{
  #add
  $oldValue = "</GeneralSettings>";  
  $newValue = "  <ExternalGuestUsersAzureAppID>" + $mwdAppId + "</ExternalGuestUsersAzureAppID>`n</GeneralSettings>"
  ((Get-Content -path $generalXml -Raw) -replace $oldValue,$newValue) | Set-Content -Path $generalXml
}

#update configuration - ExternalGuestUsersAzureAppTenantId
$sel = Select-String -Path $generalXml "<ExternalGuestUsersAzureAppTenantId>.*</ExternalGuestUsersAzureAppTenantId>"
if ($sel -ne $null)
{
  #update
  $oldValue = "<ExternalGuestUsersAzureAppTenantId>.*</ExternalGuestUsersAzureAppTenantId>"
  $newValue = "<ExternalGuestUsersAzureAppTenantId>" + $azureAdTenantIdGuid + "</ExternalGuestUsersAzureAppTenantId>"
  ((Get-Content -path $generalXml -Raw) -replace $oldValue,$newValue) | Set-Content -Path $generalXml
}
else
{
  #add
  $oldValue = "</GeneralSettings>";  
  $newValue = "  <ExternalGuestUsersAzureAppTenantId>" + $azureAdTenantIdGuid + "</ExternalGuestUsersAzureAppTenantId>`n</GeneralSettings>"
  ((Get-Content -path $generalXml -Raw) -replace $oldValue,$newValue) | Set-Content -Path $generalXml
}

#update configuration - ExternalGuestUsersAzureAppSecret
$sel = Select-String -Path $generalXml "<ExternalGuestUsersAzureAppSecret>.*</ExternalGuestUsersAzureAppSecret>"
if ($sel -ne $null)
{
  #update
  $oldValue = "<ExternalGuestUsersAzureAppSecret>.*</ExternalGuestUsersAzureAppSecret>"
  $newValue = "<ExternalGuestUsersAzureAppSecret>" + $mwdAppAccessKeyValue + "</ExternalGuestUsersAzureAppSecret>"
  ((Get-Content -path $generalXml -Raw) -replace $oldValue,$newValue) | Set-Content -Path $generalXml
}
else
{
  #add
  $oldValue = "</GeneralSettings>";  
  $newValue = "  <ExternalGuestUsersAzureAppSecret>" + $mwdAppAccessKeyValue + "</ExternalGuestUsersAzureAppSecret>`n</GeneralSettings>"
  ((Get-Content -path $generalXml -Raw) -replace $oldValue,$newValue) | Set-Content -Path $generalXml
}

"Your App ID and Tenant ID Settings should have been updated automatically in Enterprise Guest User Settings."
"To complete setup, Approve your app in the browser, then copy and paste details Application Secret manually."
""

